package com.link.core.basic.database;

import com.link.core.util.ReflectUtil;
import com.link.core.util.StringUtils;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.Date;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;

@Component
public class DataSourceAspect {
   private static final Logger logger = LogManager.getLogger(DataSourceAspect.class);

   public Object changeDataSource(ProceedingJoinPoint point) throws Throwable {
      Object target = point.getTarget();
      String method = point.getSignature().getName();
      Class<?>[] parameterTypes = ((MethodSignature)point.getSignature()).getMethod().getParameterTypes();

      try {
         DataSource data = this.getDataSource(target, method, parameterTypes);
         String dataSourceName = null;
         if (data == null) {
            dataSourceName = DataSourceType.MASTER.getName();
         } else {
            dataSourceName = data.value().getName();
         }

         DynamicDataSourceHolder.putDataSource(dataSourceName);
         if (!StringUtils.equals(dataSourceName, DataSourceType.MASTER.getName())) {
            logger.info("数据源为：" + dataSourceName);
         }
      } catch (Exception var11) {
         Exception e = var11;
         DynamicDataSourceHolder.putDataSource(DataSourceType.MASTER.getName());
         logger.error("调用方法【" + method + "】时，切换数据源失败");
         logger.error(e.getMessage());
      }

      Object var13;
      try {
         var13 = point.proceed();
      } finally {
         DynamicDataSourceHolder.clearDataSouce();
      }

      return var13;
   }

   private DataSource getDataSource(Object target, String method, Class<?>[] parameterTypes) {
      Class<?> targetClass = target.getClass();
      Method targetMethod = ReflectUtil.getMethod(targetClass, method, parameterTypes);
      if (targetMethod != null && targetMethod.isAnnotationPresent(DataSource.class)) {
         logger.debug("实现类的方法指定了数据源");
         return (DataSource)targetMethod.getAnnotation(DataSource.class);
      } else if (targetClass.isAnnotationPresent(DataSource.class)) {
         logger.debug("实现类指定了数据源");
         return (DataSource)targetClass.getAnnotation(DataSource.class);
      } else {
         return null;
      }
   }

   public static void main(String[] args) throws Exception {
      SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      Date date1 = sdf.parse("2019-10-10 19:01:01");
      Date date2 = sdf.parse("2019-10-10 19:01:02");
      System.out.println(date2.getTime() - date1.getTime());
   }
}
